//----------------------------------------------------------------------
//  ヒルベルト変換用フィルタ，次数: 4K+2, K: 整数
//      リングバッファ使用
//
//  2023/10/02, Copyright (c) 2023 MIKAMI, Naoki
//----------------------------------------------------------------------

#include "Array.hpp"
using namespace Mikami;
#pragma diag_suppress 870   // マルチバイト文字使用の警告抑制のため

#ifndef HILBERT_TRANSFORMER_HPP
#define HILBERT_TRANSFORMER_HPP

// 次数の推奨値：4K+2, K: 整数
class Hilbert
{
public:
    Hilbert(int order, const float hk[])
        : ORDER_(order), ORDER2_(order/2), hm_((order-2)/4+1, hk)
    {
        Clear();
        if ((order+1) > SIZE_)
            mbed_assert_internal("次数を 255 次以下にしてください",
                                 __FILE__, __LINE__);   // mbed_assert.h
    }
    
    // yI: 同相信号
    // yQ: 直交信号
    void Execute(float xn, float& yI, float& yQ)
    {
        __IO uint8_t n1 = index_++;
        // 現在の入力信号をリングバッファに格納
        xn_[n1] = xn;

        uint8_t n2 = n1 - ORDER_;   // 一番古いデータに対するインデックス
        uint8_t n3 = n1 - ORDER2_;  // 中央のデータに対するインデックス

        yQ = 0.0f;
        for (int k=0; k<=ORDER_/4; k++, n1-=2, n2+=2)
            yQ = yQ + hm_[k]*(xn_[n1] - xn_[n2]);
        yI = xn_[n3];               // 同相信号
    }
    
    void Clear() { for (int n=0; n<SIZE_; n++) xn_[n] = 0; }

private:
    static const int SIZE_ = 256;   // 入力信号のリングバッファのサイズ
    const int ORDER_;               // 次数
    const int ORDER2_;              // 次数の半分
    const Array<float> hm_;         // 係数
    float xn_[SIZE_];               // 入力のバッファ
    uint8_t index_;                 // 最新の入力信号を示すインデックス

    // コピー･コンストラクタ，代入演算子の禁止のため
    Hilbert(const Hilbert&);
    Hilbert& operator=(const Hilbert&);
};
#endif  // HILBERT_TRANSFORMER_HPP
